@@ -14,7 +14,6 @@
 
 import abc
 import collections
-import sys
 import threading
 
 from oslo_log import log as logging
@@ -24,16 +23,6 @@ from oslo_messaging._drivers import common
 
 LOG = logging.getLogger(__name__)
 
-# TODO(harlowja): remove this when we no longer have to support 2.7
-if sys.version_info[0:2] < (3, 2):
-    def wait_condition(cond):
-        # FIXME(markmc): timeout needed to allow keyboard interrupt
-        # http://bugs.python.org/issue8844
-        cond.wait(timeout=1)
-else:
-    def wait_condition(cond):
-        cond.wait()
-
 
 class Pool(object, metaclass=abc.ABCMeta):
     """A thread-safe object pool.
@@ -80,7 +69,7 @@ class Pool(object, metaclass=abc.ABCMeta):
             self._items.append((ttl_watch, item))
             self._cond.notify()
 
-    def get(self):
+    def get(self, retry=None):
         """Return an item from the pool, when one is available.
 
         This may cause the calling thread to block.
@@ -102,11 +91,11 @@ class Pool(object, metaclass=abc.ABCMeta):
                             "current size %s surpasses max "
                             "configured rpc_conn_pool_size %s",
                             self._current_size, self._max_size)
-                wait_condition(self._cond)
+                self._cond.wait()
 
         # We've grabbed a slot and dropped the lock, now do the creation
         try:
-            return self.create()
+            return self.create(retry=retry)
         except Exception:
             with self._cond:
                 self._current_size -= 1
@@ -122,7 +111,7 @@ class Pool(object, metaclass=abc.ABCMeta):
                 return
 
     @abc.abstractmethod
-    def create(self):
+    def create(self, retry=None):
         """Construct a new item."""
 
 
@@ -141,9 +130,9 @@ class ConnectionPool(Pool):
         LOG.debug("Idle connection has expired and been closed."
                   " Pool size: %d" % len(self._items))
 
-    def create(self, purpose=common.PURPOSE_SEND):
+    def create(self, purpose=common.PURPOSE_SEND, retry=None):
         LOG.debug('Pool creating new connection')
-        return self.connection_cls(self.conf, self.url, purpose)
+        return self.connection_cls(self.conf, self.url, purpose, retry=retry)
 
     def empty(self):
         for item in self.iter_free():
