@@ -55,8 +55,7 @@ _proton = importutils.try_import("proton")
 CYRUS_ENABLED = (pyngus and pyngus.VERSION >= (2, 0, 0) and _proton and
                  getattr(_proton.SASL, "extended", lambda: False)())
 # same with SSL
-# SSL_ENABLED = (_proton and getattr(_proton.SSL, "present", lambda: False)())
-SSL_ENABLED = False
+SSL_ENABLED = (_proton and getattr(_proton.SSL, "present", lambda: False)())
 
 LOG = logging.getLogger(__name__)
 
@@ -227,7 +226,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
                          {"msg": "value"}, wait_for_reply=False)
         self.assertIsNone(rc)
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         self.assertEqual({"msg": "value"}, listener.messages.get().message)
 
         predicate = lambda: (self._broker.sender_link_ack_count == 1)
@@ -260,9 +259,9 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
         self.assertEqual('e2', rc.get('correlation-id'))
 
         listener1.join(timeout=30)
-        self.assertFalse(listener1.isAlive())
+        self.assertFalse(listener1.is_alive())
         listener2.join(timeout=30)
-        self.assertFalse(listener2.isAlive())
+        self.assertFalse(listener2.is_alive())
         driver.cleanup()
 
     def test_messaging_patterns(self):
@@ -313,9 +312,9 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
                     {"method": "echo", "id": "fanout"})
 
         listener1.join(timeout=30)
-        self.assertFalse(listener1.isAlive())
+        self.assertFalse(listener1.is_alive())
         listener2.join(timeout=30)
-        self.assertFalse(listener2.isAlive())
+        self.assertFalse(listener2.is_alive())
         self.assertEqual(1, self._broker.fanout_count)
 
         listener1_ids = [x.message.get('id') for x in listener1.get_messages()]
@@ -359,7 +358,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
                           wait_for_reply=True,
                           timeout=1.0)
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
     def test_released_send(self):
@@ -428,7 +427,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
                           wait_for_reply=True,
                           timeout=1.0)
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
 
         predicate = lambda: (self._broker.sender_link_ack_count == 1)
         _wait_until(predicate, 30)
@@ -465,7 +464,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
                           wait_for_reply=True,
                           timeout=5.0)
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
     def test_call_reply_timeout(self):
@@ -497,7 +496,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
                           wait_for_reply=True,
                           timeout=3)
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
     def test_listener_requeue(self):
@@ -515,7 +514,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
         self.assertIsNone(rc)
 
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
 
         predicate = lambda: (self._broker.sender_link_requeue_count == 1)
         _wait_until(predicate, 30)
@@ -564,7 +563,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
         _wait_until(predicate, 30)
         self.assertTrue(predicate())
 
-        self.assertTrue(listener.isAlive())
+        self.assertTrue(listener.is_alive())
         self.assertEqual({"msg": "value"}, listener.messages.get().message)
 
         predicate = lambda: (self._broker.receiver_link_count == 0)
@@ -580,7 +579,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
         _wait_until(predicate, 30)
         self.assertTrue(predicate())
 
-        self.assertTrue(listener.isAlive())
+        self.assertTrue(listener.is_alive())
         self.assertEqual({"msg": "value"}, listener.messages.get().message)
 
         predicate = lambda: (self._broker.receiver_link_count == 0)
@@ -606,7 +605,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
         self.assertIsNotNone(rc)
         self.assertEqual("1", rc.get('correlation-id'))
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
     def test_call_monitor_bad_no_heartbeat(self):
@@ -626,7 +625,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
                           timeout=60,
                           call_monitor_timeout=5)
         listener.kill()
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
     def test_call_monitor_bad_call_timeout(self):
@@ -646,7 +645,7 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
                           timeout=11,
                           call_monitor_timeout=5)
         listener.kill()
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
 
@@ -681,7 +680,7 @@ class TestAmqpNotification(_AmqpBrokerTestCaseAuto):
                     excepted_targets.append(t)
 
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         topics = [x.message.get('target') for x in listener.get_messages()]
         self.assertEqual(msg_count, len(topics))
         self.assertEqual(2, topics.count('topic-1.info'))
@@ -763,7 +762,7 @@ class TestAuthentication(test_utils.BaseTestCase):
                          {"method": "echo"}, wait_for_reply=True)
         self.assertIsNotNone(rc)
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
     def test_authentication_failure(self):
@@ -865,7 +864,7 @@ mech_list: ${mechs}
                              retry=retry)
             self.assertIsNotNone(rc)
             listener.join(timeout=30)
-            self.assertFalse(listener.isAlive())
+            self.assertFalse(listener.is_alive())
         finally:
             driver.cleanup()
 
@@ -932,26 +931,32 @@ class TestFailover(test_utils.BaseTestCase):
         # configure different addressing modes on the brokers to test failing
         # over from one type of backend to another
         self.config(addressing_mode='dynamic', group="oslo_messaging_amqp")
-        self._brokers = [FakeBroker(self.conf.oslo_messaging_amqp,
-                                    product="qpid-cpp"),
-                         FakeBroker(self.conf.oslo_messaging_amqp,
-                                    product="routable")]
+        self._brokers = self._gen_brokers()
         self._primary = 0
         self._backup = 1
         hosts = []
         for broker in self._brokers:
             hosts.append(oslo_messaging.TransportHost(hostname=broker.host,
                                                       port=broker.port))
-        self._broker_url = oslo_messaging.TransportURL(self.conf,
-                                                       transport="amqp",
-                                                       hosts=hosts)
+        self._broker_url = self._gen_transport_url(hosts)
 
     def tearDown(self):
         super(TestFailover, self).tearDown()
         for broker in self._brokers:
-            if broker.isAlive():
+            if broker.is_alive():
                 broker.stop()
 
+    def _gen_brokers(self):
+        return [FakeBroker(self.conf.oslo_messaging_amqp,
+                           product="qpid-cpp"),
+                FakeBroker(self.conf.oslo_messaging_amqp,
+                           product="routable")]
+
+    def _gen_transport_url(self, hosts):
+        return oslo_messaging.TransportURL(self.conf,
+                                           transport="amqp",
+                                           hosts=hosts)
+
     def _failover(self, fail_broker):
         self._brokers[0].start()
         self._brokers[1].start()
@@ -1007,7 +1012,7 @@ class TestFailover(test_utils.BaseTestCase):
         self.assertEqual(1, self._brokers[self._backup].direct_count)
 
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
 
         # note: stopping the broker first tests cleaning up driver without a
         # connection active
@@ -1087,7 +1092,7 @@ class TestFailover(test_utils.BaseTestCase):
 
         listener1.join(timeout=30)
         listener2.join(timeout=30)
-        self.assertFalse(listener1.isAlive() or listener2.isAlive())
+        self.assertFalse(listener1.is_alive() or listener2.is_alive())
 
         driver.cleanup()
         self._brokers[1].stop()
@@ -1127,7 +1132,7 @@ class TestLinkRecovery(_AmqpBrokerTestCase):
             driver.cleanup()
             raise
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         self.assertEqual(listener.messages.get().message.get('method'), "echo")
         driver.cleanup()
 
@@ -1193,7 +1198,7 @@ class TestLinkRecovery(_AmqpBrokerTestCase):
                     wait_for_reply=False)
         listener.join(timeout=30)
         self.assertTrue(self._broker.fanout_count == 1)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         self.assertEqual(listener.messages.get().message.get('method'), "echo")
         driver.cleanup()
 
@@ -1208,8 +1213,8 @@ class TestLinkRecovery(_AmqpBrokerTestCase):
             else:
                 # unblock all link when RPC call is made
                 link.add_capacity(10)
-                for l in self._blocked_links:
-                    l.add_capacity(10)
+                for li in self._blocked_links:
+                    li.add_capacity(10)
 
         self._broker.on_receiver_active = _on_active
         self._broker.on_credit_exhausted = lambda link: None
@@ -1243,7 +1248,7 @@ class TestLinkRecovery(_AmqpBrokerTestCase):
         self.assertEqual(rc.get('correlation-id'), 'e1')
         listener.join(timeout=30)
         self.assertTrue(self._broker.fanout_count == 3)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
 
@@ -1293,15 +1298,15 @@ class TestAddressing(test_utils.BaseTestCase):
                                      {"msg": topic}, 2.0)
             expected.append(topic)
 
-        for l in rl:
-            l.join(timeout=30)
+        for li in rl:
+            li.join(timeout=30)
 
         # anycast will not evenly distribute an odd number of msgs
         predicate = lambda: len(expected) == (nl[0].messages.qsize() +
                                               nl[1].messages.qsize())
         _wait_until(predicate, 30)
-        for l in nl:
-            l.kill(timeout=30)
+        for li in nl:
+            li.kill(timeout=30)
 
         s1_payload = [m.message.get('msg') for m in rl[0].get_messages()]
         s2_payload = [m.message.get('msg') for m in rl[1].get_messages()]
@@ -1520,7 +1525,7 @@ class TestMessageRetransmit(_AmqpBrokerTestCase):
             self.assertEqual(rc.get('correlation-id'), 'blah')
             listener.join(timeout=30)
         finally:
-            self.assertFalse(listener.isAlive())
+            self.assertFalse(listener.is_alive())
             driver.cleanup()
 
     def test_released(self):
@@ -1548,33 +1553,42 @@ class TestMessageRetransmit(_AmqpBrokerTestCase):
 
 
 @testtools.skipUnless(SSL_ENABLED, "OpenSSL not supported")
-class TestSSL(test_utils.BaseTestCase):
+class TestSSL(TestFailover):
     """Test the driver's OpenSSL integration"""
 
     def setUp(self):
-        super(TestSSL, self).setUp()
+        self._broker = None
         # Create the CA, server, and client SSL certificates:
         self._tmpdir = tempfile.mkdtemp(prefix='amqp1')
-        files = ['ca_key', 'ca_cert', 's_key', 's_req', 's_cert', 'c_key',
-                 'c_req', 'c_cert', 'bad_cert', 'bad_req', 'bad_key']
+        files = ['ca_key', 'ca_cert', 's_key', 's_req', 's_cert', 's2_key',
+                 's2_req', 's2_cert', 'c_key', 'c_req', 'c_cert', 'bad_cert',
+                 'bad_req', 'bad_key']
         conf = dict(zip(files, [os.path.join(self._tmpdir, "%s.pem" % f)
                                 for f in files]))
         conf['pw'] = 'password'
         conf['s_name'] = '127.0.0.1'
+        conf['s2_name'] = '127.0.0.2'
         conf['c_name'] = 'client.com'
+
         self._ssl_config = conf
         ssl_setup = [
             # create self-signed CA certificate:
             Template('openssl req -x509 -nodes -newkey rsa:2048'
                      ' -subj "/CN=Trusted.CA.com" -keyout ${ca_key}'
                      ' -out ${ca_cert}').substitute(conf),
-            # create Server key and certificate:
+            # create Server keys and certificates:
             Template('openssl genrsa -out ${s_key} 2048').substitute(conf),
             Template('openssl req -new -key ${s_key} -subj /CN=${s_name}'
                      ' -passin pass:${pw} -out ${s_req}').substitute(conf),
             Template('openssl x509 -req -in ${s_req} -CA ${ca_cert}'
                      ' -CAkey ${ca_key} -CAcreateserial -out'
                      ' ${s_cert}').substitute(conf),
+            Template('openssl genrsa -out ${s2_key} 2048').substitute(conf),
+            Template('openssl req -new -key ${s2_key} -subj /CN=${s2_name}'
+                     ' -passin pass:${pw} -out ${s2_req}').substitute(conf),
+            Template('openssl x509 -req -in ${s2_req} -CA ${ca_cert}'
+                     ' -CAkey ${ca_key} -CAcreateserial -out'
+                     ' ${s2_cert}').substitute(conf),
             # create a "bad" Server cert for testing CN validation:
             Template('openssl genrsa -out ${bad_key} 2048').substitute(conf),
             Template('openssl req -new -key ${bad_key} -subj /CN=Invalid'
@@ -1599,10 +1613,30 @@ class TestSSL(test_utils.BaseTestCase):
                 self._tmpdir = None
                 self.skipTest("OpenSSL tools not installed - skipping")
 
-    def _ssl_server_ok(self, url):
-        self._broker.start()
+        super(TestSSL, self).setUp()
+
         self.config(ssl_ca_file=self._ssl_config['ca_cert'],
                     group='oslo_messaging_amqp')
+
+    def _gen_brokers(self):
+        s2_conf = self._ssl_config.copy()
+        for item in ['name', 'key', 'req', 'cert']:
+            s2_conf["s_%s" % item] = s2_conf["s2_%s" % item]
+
+        return [FakeBroker(self.conf.oslo_messaging_amqp,
+                           sock_addr=self._ssl_config['s_name'],
+                           ssl_config=self._ssl_config),
+                FakeBroker(self.conf.oslo_messaging_amqp,
+                           sock_addr=s2_conf['s_name'],
+                           ssl_config=s2_conf)]
+
+    def _gen_transport_url(self, hosts):
+        url = "amqp://%s" % (",".join(map(lambda x: "%s:%d" %
+                             (x.hostname, x.port), hosts)))
+        return oslo_messaging.TransportURL.parse(self.conf, url)
+
+    def _ssl_server_ok(self, url):
+        self._broker.start()
         tport_url = oslo_messaging.TransportURL.parse(self.conf, url)
         driver = amqp_driver.ProtonDriver(self.conf, tport_url)
         target = oslo_messaging.Target(topic="test-topic")
@@ -1615,7 +1649,7 @@ class TestSSL(test_utils.BaseTestCase):
                     wait_for_reply=True,
                     timeout=30)
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
     def test_server_ok(self):
@@ -1669,7 +1703,7 @@ class TestSSL(test_utils.BaseTestCase):
                     wait_for_reply=True,
                     timeout=30)
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
     def test_bad_server_fail(self):
@@ -1723,7 +1757,7 @@ class TestSSL(test_utils.BaseTestCase):
                     wait_for_reply=True,
                     timeout=30)
         listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
+        self.assertFalse(listener.is_alive())
         driver.cleanup()
 
     def tearDown(self):
