@@ -19,7 +19,7 @@ import testscenarios
 import oslo_messaging
 from oslo_messaging.notify import dispatcher as notify_dispatcher
 from oslo_messaging.tests import utils as test_utils
-from six.moves import mock
+from unittest import mock
 
 load_tests = testscenarios.load_tests_apply_scenarios
 
@@ -152,17 +152,24 @@ class TestDispatcherFilter(test_utils.BaseTestCase):
               context={},
               match=False)),
         ('event_type_match',
-         dict(filter_rule=dict(event_type='^instance\.create'),
+         dict(filter_rule=dict(event_type=r'^instance\.create'),
               publisher_id='compute01.manager',
               event_type='instance.create.start',
               context={},
               match=True)),
         ('event_type_nomatch',
-         dict(filter_rule=dict(event_type='^instance\.delete'),
+         dict(filter_rule=dict(event_type=r'^instance\.delete'),
               publisher_id='compute01.manager',
               event_type='instance.create.start',
               context={},
               match=False)),
+        # this is only for simulation
+        ('event_type_not_string',
+         dict(filter_rule=dict(event_type=r'^instance\.delete'),
+              publisher_id='compute01.manager',
+              event_type=['instance.swim', 'instance.fly'],
+              context={},
+              match=False)),
         ('context_match',
          dict(filter_rule=dict(context={'user': '^adm'}),
               publisher_id='compute01.manager',
@@ -206,15 +213,20 @@ class TestDispatcherFilter(test_utils.BaseTestCase):
               event_type='instance.create.start',
               context={},
               match=False)),
+        ('payload_value_none',
+         dict(filter_rule=dict(payload={'virtual_size': '2048'}),
+              publisher_id='compute01.manager',
+              event_type='instance.create.start',
+              context={},
+              match=False)),
         ('mix_match',
-         dict(filter_rule=dict(event_type='^instance\.create',
+         dict(filter_rule=dict(event_type=r'^instance\.create',
                                publisher_id='^compute',
                                context={'user': '^adm'}),
               publisher_id='compute01.manager',
               event_type='instance.create.start',
               context={'user': 'admin'},
               match=True)),
-
     ]
 
     def test_filters(self):
@@ -224,7 +236,7 @@ class TestDispatcherFilter(test_utils.BaseTestCase):
 
         dispatcher = notify_dispatcher.NotificationDispatcher(
             [endpoint], serializer=None)
-        message = {'payload': {'state': 'active'},
+        message = {'payload': {'state': 'active', 'virtual_size': None},
                    'priority': 'info',
                    'publisher_id': self.publisher_id,
                    'event_type': self.event_type,
