@@ -26,8 +26,6 @@ class TestConfigOptsProxy(test_utils.BaseTestCase):
         group = 'oslo_messaging_rabbit'
         self.config(rabbit_retry_interval=1,
                     rabbit_qos_prefetch_count=0,
-                    rabbit_max_retries=3,
-                    kombu_reconnect_delay=5.0,
                     group=group)
         dummy_opts = [cfg.ListOpt('list_str', item_type=types.String(),
                                   default=[]),
@@ -40,32 +38,42 @@ class TestConfigOptsProxy(test_utils.BaseTestCase):
         url = transport.TransportURL.parse(
             self.conf, "rabbit:///"
                        "?rabbit_qos_prefetch_count=2"
-                       "&unknown_opt=4"
-                       "&kombu_reconnect_delay=invalid_value"
                        "&list_str=1&list_str=2&list_str=3"
                        "&list_int=1&list_int=2&list_int=3"
                        "&dict=x:1&dict=y:2&dict=z:3"
                        "&bool=True"
         )
-        conf = drv_cmn.ConfigOptsProxy(self.conf, url)
+        conf = drv_cmn.ConfigOptsProxy(self.conf, url, group)
         self.assertRaises(cfg.NoSuchOptError,
                           conf.__getattr__,
                           'unknown_group')
-        self.assertTrue(isinstance(getattr(conf, group),
-                                   conf.GroupAttrProxy))
+        self.assertIsInstance(getattr(conf, group),
+                              conf.GroupAttrProxy)
         self.assertEqual(1, conf.oslo_messaging_rabbit.rabbit_retry_interval)
         self.assertEqual(2,
                          conf.oslo_messaging_rabbit.rabbit_qos_prefetch_count)
-        self.assertEqual(3, conf.oslo_messaging_rabbit.rabbit_max_retries)
-        self.assertRaises(cfg.NoSuchOptError,
-                          conf.oslo_messaging_rabbit.__getattr__,
-                          'unknown_opt')
-        self.assertRaises(ValueError,
-                          conf.oslo_messaging_rabbit.__getattr__,
-                          'kombu_reconnect_delay')
         self.assertEqual(['1', '2', '3'], conf.oslo_messaging_rabbit.list_str)
         self.assertEqual([1, 2, 3], conf.oslo_messaging_rabbit.list_int)
         self.assertEqual({'x': '1', 'y': '2', 'z': '3'},
                          conf.oslo_messaging_rabbit.dict)
         self.assertEqual(True, conf.oslo_messaging_rabbit.bool)
         self.assertEqual('default', conf.oslo_messaging_rabbit.str)
+
+    def test_not_in_group(self):
+        group = 'oslo_messaging_rabbit'
+        url = transport.TransportURL.parse(
+            self.conf, "rabbit:///?unknown_opt=4"
+        )
+        self.assertRaises(cfg.NoSuchOptError,
+                          drv_cmn.ConfigOptsProxy,
+                          self.conf, url, group)
+
+    def test_invalid_value(self):
+        group = 'oslo_messaging_rabbit'
+        self.config(kombu_reconnect_delay=5.0,
+                    group=group)
+        url = transport.TransportURL.parse(
+            self.conf, "rabbit:///?kombu_reconnect_delay=invalid_value"
+        )
+        self.assertRaises(ValueError, drv_cmn.ConfigOptsProxy, self.conf,
+                          url, group)
